import os
from django.conf import settings
from datetime import datetime

# NOTE: this is because of from running CarDocument.init() from script examples, settings is not set
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "elasticsearch_cars.settings")
from parler.utils.context import smart_override

from elasticsearch_dsl import (
    Index, Document, Date, Nested, Boolean, analyzer, InnerDoc, Completion, Keyword, Text,
    Double, Object
)
from search.analyzers import Analyzers
from search.fields import ParamInnerDoc, MultilangTextInnerDoc, MultilangKeywordInnerDoc


class CarDocument(Document):
    name = Text(
        analyzer=Analyzers.partial_edge_word_analyzer,
        search_analyzer=Analyzers.ascii_fold,
        # multifield
        fields={'raw': Keyword()}
    )
    code = Keyword()
    brand_name = Text(
        analyzer=Analyzers.partial_word_analyzer,
        search_analyzer=Analyzers.ascii_fold,
        # multifield
        fields={'raw': Keyword()}
    )
    description = Object(MultilangTextInnerDoc)
    price = Double()
    created_dt = Date()
    params = Nested(ParamInnerDoc)
    # NOTE: this is wrong, but for elasticsearch_dsl_practice purposes
    bad_params = Object(ParamInnerDoc, multi=True)

    tags = Object(MultilangTextInnerDoc)

    class Index:
        name = 'elastic-cars'

        settings = {
            "number_of_shards": 1,
            "number_of_replicas": 1
        }

    @classmethod
    def init(cls, index=None, using=None):
        # adding timestamp to new index name
        timestamp = datetime.now().strftime('%Y_%m_%d_%H_%M_%S')
        new_index_name = f"{cls._index._name}_{timestamp}"
        # register aliases
        cls._index.aliases(**{"elastic-cars": {}})
        # register analyzers
        cls._index.analyzer(Analyzers.ascii_fold)
        cls._index.analyzer(Analyzers.partial_word_analyzer)
        cls._index.analyzer(Analyzers.partial_edge_word_analyzer)

        super().init(index=new_index_name, using=using)

    def get_trans_dict(self, data, fn):
        trans_dict = {}
        for lang in dict(settings.LANGUAGES).keys():
            trans_dict[lang] = fn(data, lang)

        return trans_dict

    def add_params(self, params_instances):

        for param_instance in params_instances:
            self.params.append(
                ParamInnerDoc(name=param_instance.name_slug, value=param_instance.value_slug)
            )
            self.bad_params.append(
                ParamInnerDoc(name=param_instance.name_slug, value=param_instance.value_slug)
            )

    def add_tags(self, params_instances):
        def fn(param_instances, lang):
            return [f"{param_instance.name} {param_instance.value}" for param_instance in param_instances.language(lang)]

        self.tags = MultilangTextInnerDoc(**self.get_trans_dict(params_instances, fn))

    def add_description(self, car_instance):
        def fn(car_instance, lang):
            return car_instance.safe_translation_getter("description", language_code=lang)

        self.description = MultilangTextInnerDoc(**self.get_trans_dict(car_instance, fn))
